@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998-2009, Steven G. Johnson
+; Copyright (C) 1998-2012 Massachusetts Institute of Technology and Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -191,11 +191,6 @@
 	  (class-member? type-name (class-parent class)))
       false))
 
-(defmacro-public define-class (class-name parent . properties)
-  `(define ,class-name (make-class (quote ,class-name)
-				   ,parent
-				   ,@properties)))
-
 (define no-parent false)
 
 (define (make class . property-values)
@@ -227,15 +222,20 @@
 					       (car x)
 					       (apply vector3 x)))))
 
-(define (list-property-value-constructor name)
-  (lambda x (make-property-value-pair name x)))
+(define (list-property-value-constructor name type-name)
+  (lambda x 
+    (make-property-value-pair 
+     name 
+     (if (and (= (length x) 1) (check-type type-name (car x)))
+	 (car x)
+	 x))))
 
 (define (type-property-value-constructor type-name name)
   (cond
    ((or (eq? type-name 'vector3)  (eq? type-name 'cvector3))
     (vector3-property-value-constructor name))
    ((list-type-name? type-name)
-    (list-property-value-constructor name))
+    (list-property-value-constructor name type-name))
    (else (property-value-constructor name))))
 
 (define (post-processing-constructor post-process-func constructor)
@@ -266,3 +266,57 @@
 		  (make-derived ,derive-func)))
 
 ; ****************************************************************
+; Define classes.  A bit ugly since we support (define-property ...)
+; in the property list, but Guile 2.x doesn't allow (define ...) to
+; be used in the middle of a list expression.  So, we need to extract
+; those definitions first and duplicate some of the define-property
+; code above.
+
+(defmacro-public define-class (class-name parent . properties)
+  (let ((pdefs (map
+		(lambda (p) 
+		  (let ((name (cadr p))
+			(type-name (cadddr p)))
+		    `(define ,name
+		       (type-property-value-constructor 
+			,type-name (quote ,name)))))
+		(list-transform-positive properties 
+		  (lambda (p) (eq? (car p) 'define-property)))))
+	(ppdefs (map
+		 (lambda (p) 
+		   (let ((name (cadr p))
+			 (type-name (cadddr p))
+			 (post-process-func (list-ref p 4)))
+		     `(define ,name (post-processing-constructor
+				     ,post-process-func
+				     (type-property-value-constructor 
+				      ,type-name (quote ,name))))))
+		 (list-transform-positive properties 
+		   (lambda (p) 
+		     (eq? (car p) 'define-post-processed-property)))))
+	(props (map
+		(lambda (p)
+		  (cond
+		   ((eq? (car p) 'define-property)
+		    (let ((name (cadr p))
+			  (default (caddr p))
+			  (type-name (cadddr p))
+			  (constraints (cddddr p)))
+		      `(make-property (quote ,name) ,type-name ,default
+				      not-derived ,@constraints)))
+		   ((eq? (car p) 'define-post-processed-property)
+		    (let ((name (cadr p))
+			  (default (caddr p))
+			  (type-name (cadddr p))
+			  (post-process-func (list-ref p 4))
+			  (constraints (cdr (cddddr p))))
+		      `(make-property (quote ,name) ,type-name ,default
+				      not-derived ,@constraints)))
+		   (else p)))
+		properties)))
+  `(begin
+     ,@pdefs
+     ,@ppdefs
+     (define ,class-name (make-class (quote ,class-name)
+				   ,parent
+				   ,@props)))))
