@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998-2009, Steven G. Johnson
+; Copyright (C) 1998-2012 Massachusetts Institute of Technology and Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -24,8 +24,7 @@
 
 (define params-set-list '())
 (defmacro-public define-param (name value)
-  `(if (not (defined? (quote ,name)))
-       (define ,name ,value)))
+  `(define ,name (if (defined? (quote ,name)) ,name ,value)))
 
 (defmacro-public set-param! (name value)
   `(if (not (memq (quote ,name) params-set-list))
