@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998-2009, Steven G. Johnson
+; Copyright (C) 1998-2012 Massachusetts Institute of Technology and Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -32,15 +32,17 @@
 ; Given a list of numbers, linearly interpolates n values between
 ; each pair of numbers.
 (define (interpolate n nums)
-  (cons 
-   (car nums)
-   (fold-right
-    append '()
-    (map
-     (lambda (x y)
-       (reverse (arith-sequence y (binary/ (binary- x y) (+ n 1)) (+ n 1))))
-     (reverse (cdr (reverse nums))) ; nums w/o last value
-     (cdr nums))))) ; nums w/o first value
+  (map 
+   unary->inexact
+   (cons 
+    (car nums)
+    (fold-right
+     append '()
+     (map
+      (lambda (x y)
+	(reverse (arith-sequence y (binary/ (binary- x y) (+ n 1)) (+ n 1))))
+      (reverse (cdr (reverse nums))) ; nums w/o last value
+      (cdr nums)))))) ; nums w/o first value
 
 ; Like interpolate, except only interpolates n values *on average*
 ; between each pair of numbers.  The actual number of interpolated
@@ -51,21 +53,22 @@
     (/ (fold-left + 0 (map unary-abs (map binary- (cdr nums)
 					  (reverse (cdr (reverse nums))))))
        (length (cdr nums))))
-  (if (zero? n)
-      nums
-      (cons
-       (car nums)
-       (fold-right
-	append '()
-	(map
-	 (lambda (x y)
-	   (let ((m (inexact->exact 
-		     (+ -0.5 (* (+ n 1) (/ (unary-abs (binary- x y))
-					   meandiff))))))
-	     (reverse (arith-sequence y (binary/ (binary- x y) (+ m 1)) 
-				      (+ m 1)))))
-	 (reverse (cdr (reverse nums))) ; nums w/o last value
-	 (cdr nums)))))) ; nums w/o first value
+  (map unary->inexact
+       (if (zero? n)
+	   nums
+	   (cons
+	    (car nums)
+	    (fold-right
+	     append '()
+	     (map
+	      (lambda (x y)
+		(let ((m (inexact->exact 
+			  (+ -0.5 (* (+ n 1) (/ (unary-abs (binary- x y))
+						meandiff))))))
+		  (reverse (arith-sequence y (binary/ (binary- x y) (+ m 1)) 
+					   (+ m 1)))))
+	      (reverse (cdr (reverse nums))) ; nums w/o last value
+	      (cdr nums))))))) ; nums w/o first value
 
 ; ****************************************************************
 ; Minimization and root-finding utilities (useful in ctl scripts)
@@ -360,12 +363,13 @@
   ; bracketed the root, so do this via lazy evaluation.
   (define f-memo (memoize f))
   (define (lazy x) (if (number? x) x (x)))
-  (define ((pick-bound which?))
-    (let ((fmin-pair (f-memo x-min)) (fmax-pair (f-memo x-max)))
-      (let ((fmin (car fmin-pair)) (fmax (car fmax-pair)))
-	(if (which? fmin) x-min
-	    (if (which? fmax) x-max
-		(error "failed to bracket the root in find-root-deriv"))))))
+  (define (pick-bound which?)
+    (lambda ()
+      (let ((fmin-pair (f-memo x-min)) (fmax-pair (f-memo x-max)))
+	(let ((fmin (car fmin-pair)) (fmax (car fmax-pair)))
+	  (if (which? fmin) x-min
+	      (if (which? fmax) x-max
+		  (error "failed to bracket the root in find-root-deriv")))))))
 
   (define (in-bounds? x f df a b)
     (negative? (* (- f (* df (- x a)))
