@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998-2009, Steven G. Johnson
+; Copyright (C) 1998-2012 Massachusetts Institute of Technology and Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -21,14 +21,20 @@
 ; ****************************************************************
 ; vector3 and associated operators.  (a type to represent 3-vectors)
 
+; Guile 1.6 does not support exact->inexact on complex numbers, grrr
+(define (ctl-exact->inexact x)
+  (if (real? x) (exact->inexact x) x))
+(define (vector3->inexact v) (vector-map ctl-exact->inexact v))
+(define (vector3->exact v) (vector-map inexact->exact v))
 (define (vector3 . args)
-  (if (= (length args) 0)
-      (vector 0 0 0)
-      (if (= (length args) 1)
-	  (vector (first args) 0 0)
-	  (if (= (length args) 2)
-	      (vector (first args) (second args) 0)
-	      (vector (first args) (second args) (third args))))))
+  (vector3->inexact
+   (if (= (length args) 0)
+       (vector 0 0 0)
+       (if (= (length args) 1)
+	   (vector (first args) 0 0)
+	   (if (= (length args) 2)
+	       (vector (first args) (second args) 0)
+	       (vector (first args) (second args) (third args)))))))
 (define cvector3 vector3)
 (define (vector3? v)
   (and (vector? v)
@@ -95,6 +101,10 @@
 
 (define (unary-abs x) (if (vector3? x) (vector3-norm x) (magnitude x)))
 
+(define (unary->inexact x) (if (vector3? x) 
+			       (vector3->inexact x)
+			       (ctl-exact->inexact x)))
+
 ; ****************************************************************
 
 ; Rotating a vector (see also rotation-matrix3x3 in matrix3x3.scm):
