@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998-2009, Steven G. Johnson
+; Copyright (C) 1998-2012 Massachusetts Institute of Technology and Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,16 @@
 
 ; ****************************************************************
 
+; hackery to define material-type if it is not defined, required by
+; the fact that Guile 2.x won't allow us to put define-class inside
+; an "if" statement.
+(define-class mt-hack no-parent
+  (define-property data no-default 'SCM)) ; generic user-defined data
+(set! class-list (cdr class-list)) ; delete mt-hack from list
+(set! mt-hack (cons 'material-type (cdr mt-hack))) ; rename to material-type
 (if (not (defined? 'material-type))
-    (define-class material-type no-parent
-      (define-property data no-default 'SCM))) ; generic user-defined data
+    (set! class-list (cons mt-hack class-list)))
+(define material-type (if (defined? 'material-type) material-type mt-hack))
 
 ; A default material so that we don't have to specify a material for
 ; an object when we just care about its geometry.  If material-type is
@@ -53,6 +60,21 @@
 (define-class cone cylinder
   (define-property radius2 0 'number))
 	       
+(define-class wedge cylinder
+  (define-property wedge-angle (* 8 (atan 1)) 'number)
+  (define-property wedge-start (vector3 1 0 0) 'vector3)
+  (define-derived-property e1 'vector3
+    (lambda (object)
+      (let ((a (object-property-value object 'axis))
+	    (s (object-property-value object 'wedge-start)))
+	(unit-vector3 (vector3- s (vector3-scale (vector3-dot a s) a))))))
+  (define-derived-property e2 'vector3
+    (lambda (object)
+      (let ((a (object-property-value object 'axis))
+	    (s (object-property-value object 'start))
+	    (e1 (object-property-value object 'e1)))
+	(vector3-cross a e1)))))
+	       
 (define-class sphere geometric-object
   (define-property radius no-default 'number non-negative?))
 	       
